#include "TextEnhance.hpp"
#include "Scanner.hpp"
#include <ES2Command/ES2CommandAPI.h>
#include <utils/PathUtils.h>
#include <utils/ESFile.h>

namespace epsonscan {

static std::string GetPluginRootDir()
{
	  std::string strModulePath = EPSON_INSTALL_PATH;
      ES_CMN_FUNCS::PATH::ES_CombinePath(strModulePath, strModulePath, ES_STRING("non-free-exec"));
	  return strModulePath;
}

void TextEnhance::SetValue(const SDIInt& intVal)
{
	if (GetValueInt(kSDIColorTypeKey) == kSDIColorTypeMono1)
	{
		if (intVal == 2)
		{
			current_ = 1;
		}else{
			current_ = intVal;
		}
	}else{
		current_ = intVal;
	}
}

void TextEnhance::Reset()
{
	current_  = kSDITextEnhanceLevelNone;
}

void TextEnhance::GetValue(SDIInt& intVal)
{
	intVal = current_;
}

void TextEnhance::GetCapability(SDICapability &capability)
{
	SetDefaultListCapability(capability);
	ESIndexSet availableColorTypes;
  	capability.supportLevel = kSDISupportLevelNone;
	capability.countOfList = 0;

	ESString ddeName;
	ESString imfName;
		
	ModelInfo* modelInfo = dataProvider_->GetModelInfo().get();
		
	modelInfo->GetValue(kPrvHRD_DDEName, ddeName);
	modelInfo->GetValue(kPrvHRD_ImageFilterName, imfName);
	if (ddeName == "EsDDE6.dll" && imfName =="EsImfl6.dll")
	{
		std::string pluginRoot = GetPluginRootDir();
		ESString ddePlugin;
		ESString ddeso;
		ESString imfso;
		ES_CMN_FUNCS::PATH::ES_CombinePath(ddePlugin, pluginRoot, ES_STRING("ddeplugin"));
		ES_CMN_FUNCS::PATH::ES_CombinePath(ddeso, pluginRoot, ES_STRING("libdde6.so"));
		ES_CMN_FUNCS::PATH::ES_CombinePath(imfso, pluginRoot, ES_STRING("libimf6.so"));

		if (ES_CMN_FUNCS::PATH::ES_IsExistFile(ddePlugin) && 
			ES_CMN_FUNCS::PATH::ES_IsExistFile(ddeso) &&
			ES_CMN_FUNCS::PATH::ES_IsExistFile(imfso))
		{
			capability.supportLevel = kSDISupportLevelAvailable;
			capability.countOfAllList = 3;
			capability.allList[0] = kSDITextEnhanceLevelNone;
			capability.allList[1] = kSDITextEnhanceLevelStandard;
			capability.allList[2] = kSDITextEnhanceLevelHigh;

			capability.countOfList = 3;
			capability.list[0] = kSDITextEnhanceLevelNone;
			capability.list[1] = kSDITextEnhanceLevelStandard;
			capability.list[2] = kSDITextEnhanceLevelHigh;
		}
	}
}

ESDictionary TextEnhance::GetEngineKeyValues()
{
	auto dict =  ESDictionary();
	return dict;
}
	
}//namespace epsonscan
